==> ruby-borges-0.1.1.tbz: lib/ruby/site_ruby/1.8/borges/Renderer.rb <==
require 'borges/HTML'

module Borges

def render(&block)
  Renderer.new.render_response(&block)
end

class Renderer < HTML
  
  def render_response
    @callbacks = CallbackStore.new
    request = respond do |res|
      @response = @stream = res
      yield(self)
    end
    @callbacks.process_callbacks(request)
  end

  def open_anchor(attrs={}, &action)
    action_key = @callbacks.register_action_callback(action)
    attrs['href'] = "#{@response.action_url}?#{action_key}"
    open_tag("a", attrs)
  end
  
  def anchor(text, attrs={}, &action)
    open_anchor(attrs, &action)
    print text
    close_tag("a")
  end
  
  def form(attrs={})
    attrs["method"] ||= "POST"
    attrs["action"] = @response.action_url
    open_tag("form", attrs)
    if block_given?
      yield
      close_tag("form")
    end
  end
  
  def default_action(&action)
    action_key = @callbacks.register_action_callback(action)
    open_tag("input", {:type=>"hidden", :name=>action_key})
  end
  
  def input(type, attrs={})
    attrs["type"] = type
    open_tag("input", attrs)
  end
  
  def submit(attrs={}, &action)
    if action
      attrs["name"] = @callbacks.register_action_callback(action)
    end
    input("submit", attrs)
  end
  
  def text_input(value, attrs={}, &update)
    attrs["value"] = value
    attrs["name"] = @callbacks.register_callback(update)
    input("text", attrs)
  end
  
  def checkbox(value, attrs={}, &update)
    update_key = @callbacks.register_callback(lambda { |v|
      update.call(v != "false")
    })

    attrs["checked"] = value
    attrs["name"] = update_key
    input("checkbox", attrs)

    if value
      input("hidden", :name=>update_key, :value=>"false")
    end
  end
  
  def open_select(attrs = {})
    attrs["name"] = @callbacks.register_dispatch_callback
    open_tag("select", attrs)
  end
  
  def option(selected, attrs = {}, &update)
    attrs["selected"] = selected
    attrs["value"] = @callbacks.register_callback(update)
    open_tag("option", attrs)
  end
  
  def select(list, selection, attrs = {}, &update)
    open_select(attrs)
    list.each do |i|
      option(i == selection){update.call(i)}
      print i
    end
    close_tag("select")
  end
end

##
# CallbackStore

class CallbackStore

  ##
  # Contains a Cache for holding callbacks

  def initialize
    @cache = Cache.new
  end
  
  ##
  #

  def process_callbacks(request)
    @action = nil
    sort_request(request).each { |k,v| @cache[k.to_s].call(v) }
    @action.call if @action
  end
  
  ##
  # Store callback in the Cache.  Returns the index of the callback
  # in the cache

  def register_callback(proc)
    @cache.store(proc)
  end
  
  ##
  #

  def register_action_callback(proc)
    register_callback lambda { @action = proc }
  end

  ##
  #

  def register_dispatch_callback
    register_callback lambda { |v| @cache[v.to_i].call(nil) }
  end

  private

  ##
  # Sort requests ...

  def sort_request(request)
    request.collect {|k,v| [k.to_i, v]}.sort{|a,b| a[0] <=> b[0] }
  end
end

end

==> ruby-borges-0.2.0.tbz: lib/ruby/site_ruby/1.6/borges/Renderer.rb <==
require 'borges/HTML'

module Borges

def render(&block)
  Renderer.new.render_response(&block)
end

class Renderer < HTML
  
  ##
  # Render the page and dispatch the callback for the clicked action
  
  def render_response
    @callbacks = CallbackStore.new
    request = respond do |res|
      @response = @stream = res
      yield(self)
    end
    @callbacks.process_callbacks(request)
  end

  def open_anchor(attrs={}, &action)
    action_key = @callbacks.register_action_callback(action)
    attrs['href'] = "#{@response.action_url}?#{action_key}"
    open_tag("a", attrs)
  end
  
  def anchor(text, attrs={}, &action)
    open_anchor(attrs, &action)
    print text
    close_tag("a")
  end
  
  def form(attrs={})
    attrs["method"] ||= "POST"
    attrs["action"] = @response.action_url
    open_tag("form", attrs)
    if block_given?
      yield
      close_tag("form")
    end
  end
  
  def default_action(&action)
    action_key = @callbacks.register_action_callback(action)
    open_tag("input", {:type=>"hidden", :name=>action_key})
  end
  
  def input(type, attrs={})
    attrs["type"] = type
    open_tag("input", attrs)
  end
  
  def submit(attrs={}, &action)
    if action
      attrs["name"] = @callbacks.register_action_callback(action)
    end
    input("submit", attrs)
  end
  
  def text_input(value, attrs={}, &update)
    attrs["value"] = value
    attrs["name"] = @callbacks.register_callback(update)
    input("text", attrs)
  end

  def password_input(value, attrs={}, &update)
    attrs["value"] = value
    attrs["name"] = @callbacks.register_callback(update)
    input("password", attrs)
  end
  
  def checkbox(value, attrs={}, &update)
    update_key = @callbacks.register_callback(lambda { |v|
      update.call(v != "false")
    })

    attrs["checked"] = value
    attrs["name"] = update_key
    input("checkbox", attrs)

    if value
      input("hidden", :name=>update_key, :value=>"false")
    end
  end
  
  def open_select(attrs = {})
    attrs["name"] = @callbacks.register_dispatch_callback
    open_tag("select", attrs)
  end
  
  def option(selected, attrs = {}, &update)
    attrs["selected"] = selected
    attrs["value"] = @callbacks.register_callback(update)
    open_tag("option", attrs)
  end
  
  def select(list, selection, attrs = {}, &update)
    open_select(attrs)
    list.each do |i|
      option(i == selection){update.call(i)}
      print i
    end
    close_tag("select")
  end
end

##
# CallbackStore

class CallbackStore

  ##
  # Contains a Cache for holding callbacks

  def initialize
    @cache = Cache.new
  end
  
  ##
  # Run the action the user clicked

  def process_callbacks(request)
    @action = nil
    sort_request(request).each { |k,v| @cache[k.to_s].call(v) }
    @action.call if @action
  end
  
  ##
  # Store callback in the Cache.  Returns the index of the callback
  # in the cache

  def register_callback(proc)
    @cache.store(proc)
  end
  
  ##
  #

  def register_action_callback(proc)
    register_callback lambda { @action = proc }
  end

  ##
  #

  def register_dispatch_callback
    register_callback lambda { |v| @cache[v.to_s].call(nil) }
  end

  private

  ##
  # Sort requests ...

  def sort_request(request)
    request.collect {|k,v| [k.to_i, v]}.sort{|a,b| a[0] <=> b[0] }
  end
end

end
